


#include "MCP2515.h"

#include "SPI.h"
#include "Utilitaires.h"



/* Fonction permettant d'initialiser le  MCP2515 .
La frquence du bus CAN est configur  125 kHZ.


Paramtres d'entre :

Aucun.

Valeur de retour :

Aucune.
*/
void InitMCP2515(void)
{
	/* Envoi de la commande de  RESET , qui consiste en l'envoi de l'octet  0xC0 , par le bus SPI. */
	LireEcrireDonnesMCP2515(1, MCP2515_RESET, 0x00, 0x00);
	
	Delai(1000);	/* Dlai permettant de s'assurer que le cicruit  MCP2515  soit bien rinitialis. */



/* Passage du circuit MCP2515 en  Configuration mode , ncssaire pour pouvoir effectuer les oprations de configurations qui suivent */

	EcrireRegistreMCP2515(MCP2515_CANCTRL, 0x87);	/* Registre 0x0F.
									  - REQOP2 = 1,
									  - REQOP1 = 0,
									  - REQOP0 = 0 : le circuit est pass en  Configuration mode ,
									  - ABAT = 0 : ne pas annuler toutes les transmissons en attentes, si ncssaire,
									  - OSM : renvois des messages, si ncessaire,
									  - CLKEN = 1 : la broche  CLKOUT  est active,
									  - CLKPRE1 = 1,
									  - CLKPRE0 = 1 : rglage du prdiviseur de l'horloge de la broche  CLKOUT   8,
									  - F_CLKOUT = System Clock / 8 */

	EcrireRegistreMCP2515(MCP2515_TX01_INT, 0x3C); /* Registre 0x0C */
	EcrireRegistreMCP2515(MCP2515_TXRTSCTRL, 0x00); /* Registre 0x0D.
									L'envoi d'un message - RTS - ne se fait pas sur le passage   0  d'une broche,
									mais aprs l'envoi de la commande  RTS , par l'intermdiaire du bus SPI */


/* Initialisations des registres des compteurs d'erreurs */

	EcrireRegistreMCP2515(MCP2515_TEC, 0x00); /* Registre 0x1C.
								 Mise  zro du compteur d'erreurs en mission.*/
	EcrireRegistreMCP2515(MCP2515_REC, 0x00); /* Registre 0x1D.
								 Mise  zro du compteur d'erreurs en rception.*/


	
/* Configurations du  Bit Timing  */
							
	EcrireRegistreMCP2515(MCP2515_CNF1, 0x03 ); /* Registre 0x2A.
									 - SJW = 0 : 1 x TQ,
									 - BRP = 3. */
	EcrireRegistreMCP2515(MCP2515_CNF2, 0x90 ); /* Registre 0x29.
									 - BTLMODE = 1,
									 - SAM = 0,
									 - PHSEG1 = 2,
									 - PRSEG = 0. */
	EcrireRegistreMCP2515(MCP2515_CNF3, 0x02 ); /* Registre 0x28.
									 - SOF = 0 : Broche  CLKOUT  active pour la fonction de rptition de l'horloge,
									 - WAKFIL = 0 : Wake-up filter dsactiv,
									 - PHSEG : 2. */



/* Configurations des interruptions du circuit MCP2515 */

	EcrireRegistreMCP2515(MCP2515_CANINTE, 0x01); /* Registre 0x2B.
									 Autorisation des interruptions suite  la rception d'une donne par le module  RXB0 . */

	EcrireRegistreMCP2515(MCP2515_CANINTF, 0x00); /* Registre 0x2C.
								     Acquittement des ventuelles interruptions qui seraient en attente .*/
	
	EcrireRegistreMCP2515(MCP2515_EFLG, 0x00); /* Registre 0x2D */

	EcrireRegistreMCP2515(MCP2515_RXB0CTRL, 0x20); /* Registre 0x60 */
	
	

/* Configuration initiale du module  TXB0  */

	EcrireRegistreMCP2515(MCP2515_TXB0CTRL, 0x03); /* Registre 0x30.
								      TXP = 0b11 : les messages ont la plus haute priorit. */

	EcrireRegistreMCP2515(MCP2515_TXB0DLC, 0x01); /* Registre 0x35.
								     Nombre d'octets occups par les donnes  envoyer : 1 octet.*/


									 
/* Configurations de l'identifiant en rception par le filtre 0 du circuit MCP2515 */
		
	EcrireRegistreMCP2515(MCP2515_RXF0SIDH, 0x00);	/* Registre 0x00.
									   Identifiant initial de rception H. */

	EcrireRegistreMCP2515(MCP2515_RXF0SIDL, 0x20);	/* Registre 0x01.
									   Identifiant initial de rception L + configurations initiales du mode de fonctionnement des identifiants en rceptions. */



/* Configurations du masque de rception initial utilis par le filtre 0 du circuit MCP2515 */

	EcrireRegistreMCP2515(MCP2515_RXM0SIDH, 0x00);	/* Registre 0x20.
									   Masque de rception initial H. */
									   
	EcrireRegistreMCP2515(MCP2515_RXM0SIDL, 0x20);	/* Registre 0x21.
									   Masque de rception initial L. + configurations initiales du mode de fonctionnement des masque de rceptions. */




/* Configurations de l'identifiant en mission par le module  TXB0  du circuit MCP2515 */

	EcrireRegistreMCP2515(MCP2515_TXB0SIDH, 0x00);	/* Registre 0x31.
									   Identifiant initial d'mission H. */

	EcrireRegistreMCP2515(MCP2515_TXB0SIDL, 0x00);	/* Registre 0x32.
									   Identifiant initial d'mission L + configurations initiales du mode de fonctionnement des identifiants en missions. */

	
/* Passage du circuit MCP2515 en  Normal Operation mode  */
	
	EcrireRegistreMCP2515(MCP2515_CANCTRL, 0x07); /* Registre 0x0F.
								      - REQOP2 = 1,
								      - REQOP1 = 0,
								      - REQOP0 = 0 : le circuit est pass en  Normal Operation mode . 
								      - Les rglages prcdemment voqus restent inchangs. */

}



unsigned char LireRegistreMCP2515(unsigned char Adresse)
{
	return LireEcrireDonnesMCP2515(3, MCP2515_READ, Adresse, 0x00);	/* Envoi de 3 octets :
													- La commande d'criture ;
													- L'adresse du registre dans lequel crire ;
													- Un octet servant  gnrer l'horloge pour permettre au MCP2515 de rpondre. */
	
}

/* Fonction permettant d'crire une donne dans un registre du  MCP2515 ,
par l'intermdiaire du bus SPI


Paramtres d'entre :

unsigned char Address - Adresse du registre dans lequel crire,
unsigned char Data - Octet  crire dans le registre.

Valeur de retour :

Aucune.
*/
void EcrireRegistreMCP2515(unsigned char Adresse, unsigned char Donnee)
{
	LireEcrireDonnesMCP2515(3, MCP2515_WRITE, Adresse, Donnee);	/* Envoi de 3 octets :
													- La commande d'criture ;
													- L'adresse du registre dans lequel crire ;
													- L'octet  crire dans le registre. */
}

/* Rcupration de l'identifiant de rception reu par le module  RXB0 


Paramtres d'entre :

Aucun.

Valeur de retour :

int Identifiant - Identifiant, sur 11 bits, reu par le module  RXB0 .
*/
int RecupererIdentifiantRX0(void)
{
	unsigned char Contenu_MCP2515_RXB0SIDH;
	unsigned char Contenu_MCP2515_RXB0SIDL;

	int Identifiant;

	Contenu_MCP2515_RXB0SIDH = LireRegistreMCP2515(MCP2515_RXB0SIDH);	/* Identifiant Rception H */
	Contenu_MCP2515_RXB0SIDL = LireRegistreMCP2515(MCP2515_RXB0SIDL);	/* Identifiant Rception L  + Configurations du mode de fonctionnement des
	identifiants de rception */	
	
	Identifiant = (0x07F8 & (int)(Contenu_MCP2515_RXB0SIDH << 3));	/* On rcupre les 8 bits de poids fort de l'identifiant  recevoir,
qui sont lus tels quels dans le registre  MCP2515_RXB0SIDH . */ 
	
	Identifiant |= (0x0007 & (int)(Contenu_MCP2515_RXB0SIDL >> 5));	/* On rcupre les 3 bits de poids faible de l'identifiant  recevoir,
qui sont lus dans le registre  MCP2515_RXB0SIDL ,
aux bits  MCP2515_TXB0SIDL<5> ,  MCP2515_TXB0SIDL<6>  et  MCP2515_TXB0SIDL<7> . */

	return Identifiant;
}

/* Configuration de l'identifiant d'envoi utilis par le module  TXB0 .


Paramtres d'entre :

int Identifiant - Identifiant, sur 11 bits, que doit utiliser le module  TXB0 .

Valeur de retour :

Aucune.
*/
void ChoisirIdentifiantTX0(int Identifiant)
{
	unsigned char Contenu_MCP2515_TXB0SIDH = 0;
	unsigned char Contenu_MCP2515_TXB0SIDL = 0;

	Contenu_MCP2515_TXB0SIDH = (unsigned char)(0x00FF & (Identifiant >> 3));	/* On rcupre les 8 bits de poids fort de l'identifiant  configurer,
qui doivent tre placs tels quels dans le registre  MCP2515_TXB0SIDH . */

	Contenu_MCP2515_TXB0SIDL = (unsigned char)(0x00E0 & (Identifiant << 5));	/* On rcupre les 3 bits de poids faible de l'identifiant  configurer,
qui doivent tre placs dans le registre  MCP2515_TXB0SIDL ,
aux bits  MCP2515_TXB0SIDL<5> ,  MCP2515_TXB0SIDL<6>  et  MCP2515_TXB0SIDL<7> . */

	EcrireRegistreMCP2515(MCP2515_TXB0SIDH, Contenu_MCP2515_TXB0SIDH);	/* Identifiant Emission H */
	EcrireRegistreMCP2515(MCP2515_TXB0SIDL, Contenu_MCP2515_TXB0SIDL);	/* Identifiant Emission L + Configurations du mode de fonctionnement des
	identifiants d'mission*/
}


unsigned char RecupererTrameMCP2515(unsigned char * TamponReceptionMessage)
{
	unsigned char TailleMessageRecevoir;
	unsigned char k;
	unsigned char PointeurRegistreMCP2515 = MCP2515_RXB0D0;

	/* Lecture du nombre d'octets de donnes  recevoir, et limitation  8 octets au maximum */
	TailleMessageRecevoir = LireRegistreMCP2515(MCP2515_RXB0DLC);

	if(TailleMessageRecevoir > 8)	/* Si le nombre d'octets de donnes  recevoir dpasse le nombre maximal d'octets possible */
	{
		TailleMessageRecevoir = 8;	/* Le nombre maximal d'octets possible sera recu par le STM32F100RB */
	}

	/* Rcupration du message par le STM32F100RB */
	for(k = 0; k < TailleMessageRecevoir; k++)
	{
		TamponReceptionMessage[k] = LireRegistreMCP2515(PointeurRegistreMCP2515);	/* Les registres de donnes du module  RXB0 ,
dont les adresses sont conscutives, sont lus,
en commencant par le registre  MCP2515_RXB0D0 . */
		PointeurRegistreMCP2515++;
	}

	return TailleMessageRecevoir;
}

/* Envoi d'un message d'un maximum de 8 caractres, par le module  TXB0 .


Paramtres d'entre :

unsigned char * message - Pointeur sur le dbut d'une chane d'un maximum de 8 caractres,
 tranmettre par le module  TXB0 .

Valeur de retour :

Aucune.
*/
void EnvoyerTrameMCP2515(const unsigned char * TamponEnvoiMessage)
{
	unsigned char TailleMessageEnvoyer;
	unsigned char k;
	unsigned char PointeurRegistreMCP2515 = MCP2515_TXB0D0;

	/* Dtermination du nombre d'octets de donnes  transmettre, et limitation  8 octets au maximum */
	TailleMessageEnvoyer = Longueur(TamponEnvoiMessage);

	if(TailleMessageEnvoyer > 8)	/* Si le nombre d'octets de donnes  transmettre dpasse le nombre maximal d'octets possible */
	{
		TailleMessageEnvoyer = 8;	/* Le nombre maximal d'octets possible sera tranmis par le circuit MCP2515 */
	}

	EcrireRegistreMCP2515(MCP2515_TXB0DLC, TailleMessageEnvoyer);	/* Le MCP2515 est inform du nombre d'octets de donnes qu'il aura  tranmettre. */

	/* Chargement du message dans le MCP2515 */
	for(k = 0; k < TailleMessageEnvoyer; k++)
	{
		EcrireRegistreMCP2515(PointeurRegistreMCP2515, TamponEnvoiMessage[k]);	/* Les registres de donnes du module  TXB0 ,
dont les adresses sont conscutives, sont remplis,
en commencant par le registre  MCP2515_TXB0D0 . */
		PointeurRegistreMCP2515++;
	}

	/* Envoi de la commande  RTS , qui consiste en l'envoi de l'octet  0xC0 , par le bus SPI */
	LireEcrireDonnesMCP2515(1, MCP2515_RTS_TX0, 0x00, 0x00);
}



/* Acquittement des interruptions du circuit MCP2515.


Paramtres d'entre :

Aucun.

Valeur de retour :

Aucune.
*/
void AcquitterInterruptionsMCP2515(void)
{
	EcrireRegistreMCP2515(MCP2515_CANCTRL, 0x87);	/* Registe 0x0F.
									   Le circuit est pass en  Configuration mode . */
	EcrireRegistreMCP2515(MCP2515_CANINTF, 0x00);	/* Registre 0x2C.
									   Acquittement des interruptions du circuit MCP2515. */
	EcrireRegistreMCP2515(MCP2515_CANCTRL, 0x07);	/* Registe 0x0F.
									   Le circuit est pass en  Normal Operation mode . */
}


