

#include <stm32f10x.h>

#include "SPI.h"
#include "Utilitaires.h"


#define TAILLE_TAMPON_ENVOI_SPI1 32


void InitSPI1(void)
{
	RCC->APB2ENR |= (1 << 0);	/* Activation de l'horloge pour les fonctions alternatives d'entres - sorties. */
	RCC->APB2ENR |= (1 << 12);	/* Activation de l'horloge du SPI1. */

	GPIOA->CRL = ((GPIOA->CRL & 0xFFFFF00F) | 0x00000330);	/* Configurations des broches  PA1  et  PA2   en sortie de type  Push-Pull . */

	GPIOA->CRL = ((GPIOA->CRL & 0x0000FFFF) | 0xBBB30000);	/* Configurations des broches  PA4 ,  PA5 ,  PA6  et  PA7  pour les fonctions alternatives d'entres - sorties */
						
	SPI1->CR1 = 0x00000007;	/* Contenu du registre  SPI1->CR1  : 0b0100000001111110, ce qui correspond aux rglages indiqus ci-aprs,

							BIDIMODE = 0,
							BIDIOE = 0,
							CRCEN = 0,
							CRCNEXT = 0,
							DFF = 0, chaque transmission s'effectue sur 8 bits,
							CRCNEXT = 0, 
							RXONLY = 0,
							SSM = 0,
							LSBFIRST = 0, le bit de poids fort - le  MSB  - est ransmis en premier,
							MSTR = 1, le module du SPI1 fonctionne en tant que matre,
							CPOL = 1,
							CPHA = 1. */

	SPI1->CR1 |= (2 << 3);	/* Configuration de la vitesse du module SPI1  f PCLK / 2. */

	SPI1->CR2 |= (1 << 2);	/* SSOE = 1. */

	SPI1->CR1 |= (1 << 6);	/* Activation du module du SPI1,  n'effectuer qu'une fois que ce dernier ait t compltement configur, ce qui consiste  effectuer :

							SPE = 1 .*/
}

unsigned char LireEcrireDonnesMCP2515(unsigned char NombreDonneesEnvoyer, unsigned char Donnee1, unsigned char Donnee2, unsigned char Donnee3)
{
	unsigned char Reception;

	int Delai = 1000;

	unsigned char Donnees[3];

	Donnees[0] = Donnee1;
	Donnees[1] = Donnee2;
	Donnees[2] = Donnee3;

	while(SPI1->SR & 0x0080)	/* Bouclage tant que le module du SPI1 signale qu'il est occup */
	{
		Delai--;
		if(Delai == 0)return 0xFF;
	}
	Delai = 1000;

	GPIOA->BSRR = (1 << (MCP2515_CS + 16));	/* ~CS_MCP2515 = 0 */

	SPI1->DR = Donnees[0];	/* Envoi donne 1 */

	if(NombreDonneesEnvoyer > 1)
	{
		while(!(SPI1->SR & 0x0002))	/* TXE = 1 ? Bouclage tant que le module du SPI1 signale qu'il n'est pas prt  envoyer une nouvelle donne */
		{
			Delai--;
			if(Delai == 0)return 0xFF;
		}
		Delai = 1000;

		SPI1->DR = Donnees[1];	/* Envoi donne 2 */
		while(!(SPI1->SR & 0x0001))	/* RXNE = 1 ? Bouclage tant que le module du SPI1 signale qu'il n'a pas reu de nouvelle donne */
		{
			Delai--;
			if(Delai == 0)return 0xFF;
		}
		Delai = 1000;

		Reception |= (unsigned char)SPI1->DR;	/* Lecture donne 1 */

		if(NombreDonneesEnvoyer > 2)
		{	
			while(!(SPI1->SR & 0x0002))	/* TXE = 1 ? Bouclage tant que le module du SPI1 signale qu'il n'est pas prt  envoyer une nouvelle donne */
			{
				Delai--;
				if(Delai == 0)return 0xFF;
			}
			Delai = 1000;

			SPI1->DR = Donnees[2];	/* Envoi donne 3 */
		}

		while(!(SPI1->SR & 0x0001))	/* RXNE = 1 ? Bouclage tant que le module du SPI1 signale qu'il n'a pas reu de nouvelle donne */
		{
			Delai--;
			if(Delai == 0)return 0xFF;
		}
		Delai = 1000;

		Reception = (unsigned char)SPI1->DR;	/* Lecture donne 2 */

		if(NombreDonneesEnvoyer > 2)
		{
			while(!(SPI1->SR & 0x0001))	/* RXNE = 1 ? Bouclage tant que le module du SPI1 signale qu'il n'a pas reu de nouvelle donne */
			{
				Delai--;
				if(Delai == 0)return 0xFF;
			}
			Delai = 1000;

			Reception = (unsigned char)SPI1->DR;	/* Lecture donne 3 */
		}
	}

	while(SPI1->SR & 0x0080)	/* BSY = 1 ? Bouclage tant que le module du SPI1 signale qu'il est occup */
	{
		Delai--;
		if(Delai == 0)return 0xFF;
	}

	GPIOA->BSRR = (1 << MCP2515_CS);	/* ~CS_MCP2515 = 1 */

	return Reception;
}


